;; Copyright  William F. Schelter.   1994
;; Licensed by GNU public license.

;; This file contains function find-ansi-doc which finds documentation in the
;; standard common lisp ansi documentation (1350 pages!), and puts it on
;; the screen at the correct page using xdvi.  If there is more than one
;; reference it successively finds them.  You need dpANS2/*.dvi
;; dpANS2/index.idx from parcftp.xerox.com (13.1.64.94) You also need
;; xdvi.   You may gzip the .dvi files and it will unzip them into tmp
;; as needed.

(eval-and-compile
  (require 'cl-lib))

(defvar ansi-doc-dir "/usr/local/doc/dpANS2")
(defvar ansi-doc-alist nil)

(defun create-index-el-from-index-idx ()
  (interactive)
  (let (tem)
    (cond ((not ansi-doc-alist)
	   (setq tem (concat ansi-doc-dir "/index.el"))
	   (or (file-exists-p tem)
	       (progn
		 (shell-command
		  (concat "echo '(setq ansi-doc-alist (quote (( ' > " tem))
		 (shell-command
		  (concat "cat " ansi-doc-dir "/index.idx "
			  "| sed "
			  " -e 's/\\!9\\([A-Z]\\):\\([^\\!]*\\)\\!\\!/)(\"\\2\" \\1/g' "
			  " -e 's:{$\\\\spLT \\$}:<:g' "
			  " -e 's:{$\\\\spGT $}:>:g' "
			  " -e 's:\\\\&:\\&:g' "
			  " -e 's:\\([0-9]\\),:\\1:g'"
			  " -e 's:\\([A0-9][0-9]*\\)--\\([0-9][0-9]*\\):(\\1 . \\2):g'"
			  " | sort -r  "
			  " >> " tem))
		 (shell-command (concat "echo '))))' >>  " tem))))
	   
	   ))))
(defun maybe-gzip-to-tmp (file &optional dir)
  "If file exists with  .gz  added to it, then unzip it to /tmp and
return that file otherwise return file"
  (let (tmp-file)
    (cond  ((file-exists-p (concat file ".gz"))
	    (setq tmp-file
			  (file-name-nondirectory file))
	    (or (file-exists-p tmp-file)
		(progn (message "gzipping %s in /tmp for future use" file)
		       (shell-command (concat "gzip -dc < " file ".gz > "
					      tmp-file ))))
	    tmp-file)
	   (t file))))

(defun find-ansi-doc ()
  "Find the documentation in the ansi draft on a particular function
or topic.   If there are several pieces of documentation then go through
them successively.   Requires copying the "
  (interactive )
  (unless ansi-doc-alist
    (create-index-el-from-index-idx )
    (load (concat ansi-doc-dir "/index.el")))
  (let* ((name (completing-read "Doc on: " ansi-doc-alist nil t))
	 (tem (mapcan #'(lambda (elem)
			  (and (equal name (car elem)) (cddr elem)))
		      ansi-doc-alist))
	 (first (if (cdr tem) "First" "")))
    (dolist (x tem)
      (message "%s Doc in Chapter %s page %s) .." first (car x) (cdr x))
      (let ((chap (maybe-gzip-to-tmp
		   (concat ansi-doc-dir
			   (downcase (format "/chap-%s.dvi"  (car x)))))))
	(setq first (if (cdr tem) "Next" "Final"))
	(shell-command (concat "xdvi  -expert -xoffset .2 -yoffset -.2 "
			       " -paper 7.2x8.5 "
			       " -display "
			       (or
				(if (featurep 'xemacs)
				    (console-connection (selected-console))
				  x-display-name)
				":0")
			       "  -geometry -2-2 +" (+ (cdr x) 2)" "
			       chap)))))
  (message nil))
